@@ -162,9 +162,12 @@ ctdataupdate<-function(forcerecompile=FALSE){
   # 
   # checkm$pars[c(-1,-2, -21,-22) ,c('TI1_effect','TI2_effect','TI3_effect')] <- FALSE
   
-  ctstantestfit<-ctStanFit(ctstantestdat,checkm,iter=300, warmup=260,thin=2,chains=2,
-    # plot=TRUE,
-    forcerecompile=forcerecompile,save_warmup=FALSE,savescores=FALSE,
+  ctstantestfit<-ctStanFit(ctstantestdat,checkm,
+    optimize = TRUE,optimcontrol=list(finishsamples=20),
+    iter=300, warmup=260,thin=2,chains=2,
+    plot=TRUE,
+    # forcerecompile=forcerecompile,
+    save_warmup=T,savescores=FALSE,
     control=list(max_treedepth=8,adapt_delta=.8))
   ctstantestfit <- ctStanGenerateFromFit(ctstantestfit,nsamples = 20,fullposterior = TRUE)
   print( summary(ctstantestfit))
